<?php
/**
 * @file
 * Various pages created by Paypal API
 */

/**
 *
 * Generate the page for paying for a given transaction.
 *
 * @see theme_paypal_api_payment_button()
 *
 * @param $transaction_id
 * A transaction id
 *
 * @return
 * A page.
 */
function paypal_api_payment($transaction_id) {
  return theme('paypal_api_payment_page', array('transaction' => $transaction_id));
}

/**
 *
 * Generate the success and cancel feedback page.
 *
 * @param $transaction_id
 * A transaction id
 *
 * @return
 * A page.
 */
function paypal_api_success_page($transaction_id) {
  return theme('paypal_api_success_page', array('transaction' => $transaction_id));
}
function paypal_api_cancel_page($transaction_id) {
  return theme('paypal_api_cancel_page', array('transaction' => $transaction_id));
}

/**
 * Create the interim form to collect an anonymous user's email address, if necessary.
 *
 * The payment form happens in two steps:
 *
 * Step 1. Present payment information and the Paypal button, and if necessary, a text field to retrieve an anonymous
 *          user's email address
 *
 * Step 2. When the user clicks the Paypal "Pay now" button, any anonymous user's email address is stored with the transaction
 *          and then the user is automatically sent to Paypal through a Javascript click on the submit button.  Users with Javascript
 *          disabled will need to manually click a second time
 *
 * @param $form
 *
 * @param $form_state
 *
 * @param $transaction
 */
function paypal_api_payment_form($form, $form_state, $transaction) {
  if (!empty($form_state['storage']) && !empty($form_state['storage']['paypal_ready'])) {
    return paypal_api_payment_form_redirect($transaction);
  }
  else {
    return paypal_api_payment_form_confirm($transaction);
  }
}

/**
 * Step 1 of the payment process
 */
function paypal_api_payment_form_confirm($transaction) {
  global $user;

  $form = array(
    'transaction_id'  => array(
      '#type'   => 'hidden',
      '#value'  => $transaction->transaction_id
    )
  );

  $settings = paypal_api_settings();

  if (!$user->uid && $settings['create_user']) {
    $default  = (!empty($transaction->anon_email))? $transaction->anon_email : '';
    $override  = (!empty($transaction->override_email))? $transaction->override_email : FALSE;

    if ($default && !$override) {
      $form['anon_email'] = array(
        '#type'   => 'hidden',
        '#value'  => $default
      );
    }
    else {
      $form['anon_email'] = array(
        '#type'         => 'textfield',
        '#title'        => t("Your email address"),
        '#description'  => t("An account on this site is required for payment tracking.  Please give us a valid email address, which will be used to create your account.  You will receive an email with further instructions at this address."),
        '#default_value'  => $default
      );
    }
  }

  $form['submit'] = array(
    '#type'   => 'submit',
    '#value'  => t("Confirm order and pay now"),

    '#attributes' => array(
      'class'  => array('paypal-button')
    )
  );

  return $form;
}

/**
 * Validate Step 1 of the payment process
 */
function paypal_api_payment_form_validate(&$form, &$form_state) {
  global $user;
  $settings = paypal_api_settings();

  if ($user->uid || !$settings['create_user']) {
    // nothing to validate
    return;
  }

  $email = (empty($form_state['values']['anon_email']))? '' : trim($form_state['values']['anon_email']);
  if (!$email) {
    form_set_error('anon_email', t("Unregistered users must supply an email address before payment"));
  }

  if (!valid_email_address($form_state['values']['anon_email'])) {
    form_set_error('anon_email', t("'@email' doesn't look like a valid email address", array('@email' => $form_state['values']['anon_email'])));
  }
}

/**
 * Submit handler for Step 1 of the payment process
 * Save anonymous user's email address to the database for this transaction
 */
function paypal_api_payment_form_submit(&$form, &$form_state) {
  global $user;
  $settings = paypal_api_settings();

  $transaction = paypal_api_transaction($form_state['values']['transaction_id']);
  if (!$transaction) {
    drupal_set_message(t("There was an error processing this transaction.  Please contact a site administrator."), 'error');
    return;
  }

  if (!$user->uid && $settings['create_user']) {
    $transaction->anon_email = $form_state['values']['anon_email'];
    paypal_api_save_transaction($transaction);
  }

  // indicate we're good to transfer to Paypal
  $form_state['storage']['paypal_ready'] = TRUE;
  $form_state['rebuild'] = TRUE;
}

/**
 * Step 2 of the payment process
 * Create the actual Paypal form. Javascript will hide the button and automatically click it if enabled.
 *
 * @param $transaction
 *
 * @return
 * HTML for a Paypal payment form.
 */
function paypal_api_payment_form_redirect($transaction) {
  global $language;

  $values = (array) $transaction;

  // remove non-paypal fields
  unset($values['transaction_id']);
  unset($values['anon_email']);
  unset($values['override_email']);
  unset($values['uid']);

  // display and checkout fields need to be moved into the top level for proper inclusion in the form
  foreach (array('display', 'checkout') as $field) {
    if (!empty($values[$field])) {
      foreach ($values[$field] as $pp_name => $pp_value) {
        $values[$pp_name] = $pp_value;
      }
    }
    unset($values[$field]);
  }

  if (empty($values['notify_url'])) {
    $values['notify_url'] = 'paypal/ipn/' . $transaction->transaction_id;
  }
  if (empty($values['return'])) {
    $values['return'] = 'paypal/success/' . $transaction->transaction_id;
  }
  if (empty($values['cancel_return'])) {
    $values['cancel_return'] = 'paypal/cancel/' . $transaction->transaction_id;
  }

  $values['business'] = paypal_api_account_id($transaction);
  $values['cmd']      = '_xclick'; // only doing Buy Now buttons

  $form = array(
    '#action' => "https://" . paypal_api_url()
  );

  foreach ($values as $name => $value) {
    if ($value) {
      if (in_array($name, array('notify_url', 'return', 'cancel_return'))) {
        $value = paypal_api_prefix_url($value);
      }

      $form[$name] = array(
        '#type' => 'hidden',
        '#value' => $value
      );
    }
  }

  $lang_code = ($language->language == 'en')? 'en_US' : $language->language;
  $form['submit'] = array(
    '#type'   => 'image_button',
    '#src'    => 'https://www.paypal.com/' . $lang_code . '/i/btn/btn_buynow_LG.gif',
    '#value'  => t("Buy now")
  );

  drupal_add_js(drupal_get_path('module', 'paypal_api') . '/js/paypal-api-form.js');
  return $form;
}

/**
 *
 * Generate the page displayed after a successful payment.
 *
 * @see theme_paypal_api_success_page()
 *
 * @param $transaction_id
 * A transaction id
 *
 * @return
 * A page.
 */
function paypal_api_success($transaction_id) {
  return theme('paypal_api_success_page', array('transaction' => $transaction_id));
}

/**
 *
 * Generate the page displayed after a cancelled payment.
 *
 * @see theme_paypal_api_cancel_page()
 *
 * @param $transaction_id
 * A transaction id
 *
 * @return
 * A page.
 */
function paypal_api_cancel($transaction_id) {
  return theme('paypal_api_cancel_page', array('transaction' => $transaction_id));
}
